package org.rcisoft.alibaba.faceVerify.service.impl;

import com.alibaba.fastjson.JSON;
import com.aliyun.cloudauth20190307.Client;
import com.aliyun.cloudauth20190307.models.DescribeFaceVerifyRequest;
import com.aliyun.cloudauth20190307.models.DescribeFaceVerifyResponse;
import com.aliyun.cloudauth20190307.models.InitFaceVerifyRequest;
import com.aliyun.cloudauth20190307.models.InitFaceVerifyResponse;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.models.RuntimeOptions;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.alibaba.faceVerify.bean.FaceVerifyAliComp;
import org.rcisoft.alibaba.faceVerify.bean.FaceVerifyRedisBean;
import org.rcisoft.alibaba.faceVerify.service.FaceVerifyService;
import org.rcisoft.business.memGoldCoinFlow.dao.MemGoldCoinFlowRepository;
import org.rcisoft.business.memGoldCoinFlow.entity.MemGoldCoinFlow;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.entity.MemInfo;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.CyAESUtils;
import org.rcisoft.core.util.CyAddressUtils;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.util.TimeUtil;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.impl.DictionaryServiceImpl;
import org.rcisoft.sys.rbac.user.dao.SysUserRbacRepository;
import org.rcisoft.sys.rbac.user.dto.SysUserRbacDTO;
import org.rcisoft.sys.rbac.user.entity.SysUserRbac;
import org.rcisoft.sys.rbac.user.enums.UserInfoExceptionEnums;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.math.BigInteger;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Service("faceVerifyServiceImplNew")
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class FaceVerifyServiceImpl extends ServiceImpl<MemInfoRepository, MemInfo> implements FaceVerifyService {

    @Autowired
    private FaceVerifyAliComp faceVerifyAliComp;
    @Autowired
    private MemInfoRepository memInfoRepository;
    @Autowired
    private CyRedisService cyRedisServiceImpl;
    @Autowired
    private SysUserRbacRepository sysUserRbacRepository;
    @Autowired
    private DictionaryServiceImpl dictionaryService;
    @Autowired
    private MemGoldCoinFlowRepository memGoldCoinFlowRepository;
    /**
     * 发起认证请求
     * @param memInfo
     * @return
     */
    @Override
    public Map<String,String> initFaceVerify(HttpServletRequest httpServletRequest,MemInfo memInfo) {
        Object detail = this.checkIdCard(memInfo);
        if (detail!=null) {
            MemInfo info = JSON.parseObject(JSON.toJSONString(detail), MemInfo.class);
            String idCard = info.getMemIdcard();
            String name = info.getMemRealName();
            Map<String, String> map = new HashMap<>();
            InitFaceVerifyRequest request = new InitFaceVerifyRequest();
            // 场景ID+L。
            request.setSceneId(faceVerifyAliComp.getSceneId());
            // 设置商户请求的唯一标识。（请自己生成一个不超过32位数的数字及字母字符串）
            LocalDateTime now = LocalDateTime.now();
            String timestamp = now.format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss"));
            String uuid = UUID.randomUUID().toString().replace("-", "");
            String outerOrderNo = timestamp + uuid.substring(0, 14);
            request.setOuterOrderNo(outerOrderNo);
            // 认证方案。
            request.setProductCode("ID_PRO");
            // 模式。LIVENESS（默认）：眨眼动作活体检测。
            request.setModel("LIVENESS");
            // 不同证件类型，取值均为IDENTITY_CARD。
            request.setCertType("IDENTITY_CARD");
            // 您的终端用户的真实姓名。
            request.setCertName(name);
            // 您的终端用户的证件号码。
            request.setCertNo(idCard);
            // MetaInfo环境参数，此参数应由前端js获取并传入。
            request.setMetaInfo(memInfo.getMetaInfo());
            // 认证结束后回跳页面的链接地址。。
            request.setReturnUrl(faceVerifyAliComp.getReturnUrl()+"?userId="+CyUserUtil.getAuthenBusinessId());
            request.setCallbackUrl(faceVerifyAliComp.getCallbackUrl());
            request.setCallbackToken(CyAESUtils.encrypt(CyUserUtil.getAuthenBusinessId()));
            // 原始长链
            request.setCertifyUrlStyle("L");
            InitFaceVerifyResponse response = initFaceVerifyAutoRoute(request);
            if ("200".equals(response.getBody().getCode())) {
                map.put("requestId", response.getBody().getRequestId());
                map.put("certifyId", response.getBody().getResultObject().getCertifyId());
                map.put("certifyUrl", response.getBody().getResultObject().getCertifyUrl());
            } else {
                // 人脸识别异常处理
                this.faceVerifyFail(idCard);
                if ("401".equals(response.getBody().getCode())) {
                    log.error("发起认证请求失败:" + response.getBody().getMessage());
                    throw new CyServiceException("输入的姓名或者身份证号码有误");
                }  else if ("418".equals(response.getBody().getCode())) {
                    log.error("发起认证请求失败:" + response.getBody().getMessage());
                    throw new CyServiceException("当天刷脸认证次数过多，请明天再试。");
                } else {
                    log.error("发起认证请求失败:" + response.getBody().getMessage());
                    throw new CyServiceException("发起认证请求失败");
                }
            }
            return map;
        } else
            throw new CyServiceException("参数错误，请重新发起验证");
    }


    private  InitFaceVerifyResponse initFaceVerifyAutoRoute(InitFaceVerifyRequest request) {
        // 第一个为主区域Endpoint，第二个为备区域Endpoint。
        List<String> endpoints = List.of(faceVerifyAliComp.getEndpoints());
        InitFaceVerifyResponse lastResponse = null;
        for (int i=0; i<endpoints.size(); i++) {
            try {
                InitFaceVerifyResponse response = initFaceVerify(endpoints.get(i), request);
                lastResponse = response;

                // 服务端错误，切换到下个区域调用。
                if(response != null){
                    if(500 == response.getStatusCode()){
                        continue;
                    }
                    if(response.getBody() != null){
                        if("500".equals(response.getBody().getCode())){
                            continue;
                        }
                    }
                }

                // 正常返回
                return lastResponse;
            }catch (Exception e) {
                e.printStackTrace();
                if(i == endpoints.size()-1){
                    throw new RuntimeException(e);
                }
            }
        }

        return lastResponse;
    }

    private InitFaceVerifyResponse initFaceVerify(String endpoint, InitFaceVerifyRequest request)
            throws Exception {
        // 阿里云账号AccessKey拥有所有API的访问权限，建议您使用RAM用户进行API访问或日常运维。
        // 强烈建议不要把AccessKey ID和AccessKey Secret保存到工程代码里，否则可能导致AccessKey泄露，威胁您账号下所有资源的安全。
        // 本示例通过阿里云Credentials工具从环境变量中读取AccessKey，来实现API访问的身份验证。如何配置环境变量，请参见https://help.aliyun.com/document_detail/378657.html。
//        com.aliyun.credentials.Client credentialClient = new com.aliyun.credentials.Client();
        Config config = new Config();
//        config.setCredential(credentialClient);
        config.setEndpoint(endpoint);
        config.setAccessKeyId(faceVerifyAliComp.getAccessKeyId());
        config.setAccessKeySecret(faceVerifyAliComp.getAccessKeySecret());
        // 设置http代理。
        //config.setHttpProxy("http://xx.xx.xx.xx:xxxx");
        // 设置https代理。
        //config.setHttpsProxy("https://xx.xx.xx.xx:xxxx");
        Client client = new Client(config);

        // 创建RuntimeObject实例并设置运行参数。
        RuntimeOptions runtime = new RuntimeOptions();
        runtime.readTimeout = 10000;
        runtime.connectTimeout = 10000;

        return client.initFaceVerifyWithOptions(request, runtime);
    }



    /**
     * 获取认证详细数据
     * @param dto
     * @return
     */
    @Override
    @Transactional
    public  Map<String,String> describeFaceVerify(InitFaceVerifyRequest dto) {
        String userId = dto.getUserId();
        if (userId == null){
            throw new CyServiceException("系统异常，请稍后重试");
        }
        Map<String, String> map = new HashMap<>();
        Object detail = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_USER + userId);
        if (detail == null){
            throw new CyServiceException("系统异常，请稍后重试");
        }
        MemInfo info = JSON.parseObject(JSON.toJSONString(detail), MemInfo.class);
        MemInfo memInfo = memInfoRepository.getInfoByUserId(userId);
        String idCard = info.getMemIdcard();
        String realName = info.getMemRealName();
        if("1".equals(memInfo.getMemRealAuthen())){
            // 已经实名认证完了 加次数
            this.faceVerifyFail(idCard);
            throw new CyServiceException("用户已实名，无需再次实名认证");
        }
        // 创建API请求并设置参数。
        DescribeFaceVerifyRequest request = new DescribeFaceVerifyRequest();
        // 场景ID+L。
        request.setSceneId(faceVerifyAliComp.getSceneId());
        // CertifyId在InitFaceVerify接口的返回值中。
        request.setCertifyId(dto.getCertifyId());
        DescribeFaceVerifyResponse response = describeFaceVerifyAutoRoute(request);
        map.put("requestId", response.getBody().getRequestId());
        map.put("message", response.getBody().getMessage());
        if ("200".equals(response.getBody().getCode())) {
            log.info("passed:" + response.getBody().getResultObject().getPassed());
            log.info("subCode:" + response.getBody().getResultObject().getSubCode());
            map.put("passed", response.getBody().getResultObject().getPassed());
            map.put("subCode", response.getBody().getResultObject().getSubCode());
            if ("F".equals(response.getBody().getResultObject().getPassed())){
                // 识别失败 异常处理
                this.faceVerifyFail(idCard);
                throw new CyServiceException("实名认证未通过，请重试");
            }
            //身份证验重
            SysUserRbacDTO sysUserRbacDTO = baseMapper.checkIdcardRepeat(info.getUserId(),CyAESUtils.encrypt(info.getMemIdcard()));
            if (sysUserRbacDTO != null) {
                // 加次数
                this.faceVerifyFail(idCard);
                throw new CyServiceException(UserInfoExceptionEnums.ID_NUMBER_EXISTS);
            }
            List<DictData> userConfig = dictionaryService.selectByTypes("user_config");
            DictData dictData = userConfig.stream().filter(item -> item.getDictLabel().equals("real_auth_coin_count")).findFirst().orElse(null);
            Integer coinCount = Integer.valueOf(dictData.getDictValue());
            memInfo.setMemRealAuthen("1");
            memInfo.setMemRealAuthenDate(new Date());
            //根据身份证号获取性别
            memInfo.setMemSex(Integer.valueOf(idCard.substring(16, 17)) % 2 == 0 ? "1" : "0");
            //截取idCard身份证号 获取出生年月日20021029 变成2002-10-29
            String birth = idCard.substring(6, 14);
            String birthDate = birth.substring(0, 4) + "-" + birth.substring(4, 6) + "-" + birth.substring(6);
            memInfo.setMemBirthday(birthDate);
            memInfo.setMemConstellation(getConstellation(birthDate));
            memInfo.setMemIdcard(CyAESUtils.encrypt(idCard));
            memInfo.setMemRealName(CyAESUtils.encrypt(realName));
            memInfo.setGoldCoinsCount(coinCount);
            memInfoRepository.updateById(memInfo);
            //修改sys_user
            SysUserRbac userRbac = new SysUserRbac(memInfo.getUserId());
            userRbac.setIdNumber(memInfo.getMemIdcard());
            sysUserRbacRepository.updateById(userRbac);
            //增加金币流水记录
            MemGoldCoinFlow memGoldCoinFlow = new MemGoldCoinFlow();
            memGoldCoinFlow.setCreateBy(String.valueOf(1));
            memGoldCoinFlow.setUpdateBy(String.valueOf(1));
            memGoldCoinFlow.setCount(coinCount);
            memGoldCoinFlow.setEndCount(coinCount);
            memGoldCoinFlow.setType("1");
            memGoldCoinFlow.setActionType("12");
            memGoldCoinFlow.setTargetId(BigInteger.valueOf(Integer.valueOf(userId)));
            memGoldCoinFlowRepository.insert(memGoldCoinFlow);
        } else {
            log.error("获取认证详细数据失败:" + response.getBody().getMessage());
            throw new CyServiceException(response.getBody().getMessage());
        }
        map.put("realAuthen", memInfo.getMemRealAuthen());
        return map;
    }

    private DescribeFaceVerifyResponse describeFaceVerifyAutoRoute(DescribeFaceVerifyRequest request) {
        // 第一个为主区域Endpoint，第二个为备区域Endpoint。
        List<String> endpoints = List.of(faceVerifyAliComp.getEndpoints());
        DescribeFaceVerifyResponse lastResponse = null;
        for (int i = 0; i < endpoints.size(); i++) {
            try {
                DescribeFaceVerifyResponse response = describeFaceVerify(endpoints.get(i), request);
                lastResponse = response;

                // 服务端错误，切换到下个区域调用。
                if (response != null) {
                    if (500 == response.getStatusCode()) {
                        continue;
                    }
                    if (response.getBody() != null) {
                        if ("500".equals(response.getBody().getCode())) {
                            continue;
                        }
                    }
                }

                return lastResponse;
            } catch (Exception e) {
                if (i == endpoints.size() - 1) {
                    throw new RuntimeException(e);
                }
            }
        }

        return lastResponse;
    }

    private DescribeFaceVerifyResponse describeFaceVerify(String endpoint, DescribeFaceVerifyRequest request)
            throws Exception {
        // 阿里云账号AccessKey拥有所有API的访问权限，建议您使用RAM用户进行API访问或日常运维。
        // 强烈建议不要把AccessKey ID和AccessKey Secret保存到工程代码里，否则可能导致AccessKey泄露，威胁您账号下所有资源的安全。
        //本示例通过阿里云Credentials工具从环境变量中读取AccessKey，来实现API访问的身份验证。如何配置环境变量，请参见https://help.aliyun.com/document_detail/378657.html。
//        com.aliyun.credentials.Client credentialClient = new com.aliyun.credentials.Client();
        Config config = new Config();
//        config.setCredential(credentialClient);
        config.setEndpoint(endpoint);
        config.setAccessKeyId(faceVerifyAliComp.getAccessKeyId());
        config.setAccessKeySecret(faceVerifyAliComp.getAccessKeySecret());
        Client client = new Client(config);

        RuntimeOptions runtime = new RuntimeOptions();
        runtime.readTimeout = 10000;
        runtime.connectTimeout = 10000;

        return client.describeFaceVerifyWithOptions(request, runtime);
    }



    @Override
    public void callbackFaceVerify(String callbackToken, String certifyId, String passed) {
        log.info("回调callbackToken:"+callbackToken);
        log.info("回调certifyId:"+certifyId);
        log.info("回调passed:"+passed);
    }

    public static String getConstellation(String birth) {
        int month = Integer.parseInt(birth.substring(5, 7));
        int day = Integer.parseInt(birth.substring(8, 10));

        if ((month == 3 && day >= 21) || (month == 4 && day <= 19)) {
            return "3";//白羊座
        } else if ((month == 4 && day >= 20) || (month == 5 && day <= 20)) {
            return "4";//金牛座
        } else if ((month == 5 && day >= 21) || (month == 6 && day <= 21)) {
            return "5";//双子座
        } else if ((month == 6 && day >= 22) || (month == 7 && day <= 22)) {
            return "6";//巨蟹座
        } else if ((month == 7 && day >= 23) || (month == 8 && day <= 22)) {
            return "7";//狮子座
        } else if ((month == 8 && day >= 23) || (month == 9 && day <= 22)) {
            return "8";//处女座
        } else if ((month == 9 && day >= 23) || (month == 10 && day <= 23)) {
            return "9";//天秤座
        } else if ((month == 10 && day >= 24) || (month == 11 && day <= 22)) {
            return "10";//天蝎座
        } else if ((month == 11 && day >= 23) || (month == 12 && day <= 21)) {
            return "11";//射手座
        } else if ((month == 12 && day >= 22) || (month == 1 && day <= 19)) {
            return "12";//摩羯座
        } else if ((month == 1 && day >= 20) || (month == 2 && day <= 18)) {
            return "1";//水瓶座
        } else {
            // 默认返回双鱼座
            return "2";//双鱼座
        }
    }

    // 人脸识别异常处理 加次数
    public void faceVerifyFail(String idCard){
        HttpServletRequest request1 = ((ServletRequestAttributes) (RequestContextHolder.getRequestAttributes())).getRequest();
        String ip = CyAddressUtils.IpUtils.getIpAddr(request1);
        Object ipMin = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IP_MINUTES + ip);
        // 1 设置 ip 每分钟的失败次数
        if (ipMin != null) {
            int count = (int)ipMin;
            cyRedisServiceImpl.setValueNotTime(FaceVerifyRedisBean.FACEVERIFY_IP_MINUTES + ip, count+1, 0);
        } else {
            boolean set = cyRedisServiceImpl.set(FaceVerifyRedisBean.FACEVERIFY_IP_MINUTES + ip, 1, 60L);
            log.info("设置:"+set);
        }
        Object ipDay = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IP_DAY + ip);
        // 1 设置 ip 每天的失败次数
        if (ipDay != null) {
            int count = (int)ipDay;
            cyRedisServiceImpl.setValueNotTime(FaceVerifyRedisBean.FACEVERIFY_IP_DAY + ip, count+1, 0);
        } else {
            long time = TimeUtil.getTimeSecond(new Date(), TimeUtil.getTodayEndTime());
            cyRedisServiceImpl.set(FaceVerifyRedisBean.FACEVERIFY_IP_DAY + ip, 1, time);
        }
        Object idCardMin = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IDCARD_MINUTES + idCard);
        // 1 设置 身份证号 每分钟的失败次数
        if (idCardMin != null) {
            int count = (int)idCardMin;
            cyRedisServiceImpl.setValueNotTime(FaceVerifyRedisBean.FACEVERIFY_IDCARD_MINUTES + idCard, count+1, 0);
        } else {
            cyRedisServiceImpl.set(FaceVerifyRedisBean.FACEVERIFY_IDCARD_MINUTES + idCard, 1, 60L);
        }
        Object idCardDay = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IDCARD_DAY + idCard);
        // 1 设置 身份证号 每天的失败次数
        if (idCardDay != null) {
            int count = (int)idCardDay;
            cyRedisServiceImpl.setValueNotTime(FaceVerifyRedisBean.FACEVERIFY_IDCARD_DAY + idCard, count+1, 0);
        } else {
            long time = TimeUtil.getTimeSecond(new Date(), TimeUtil.getTodayEndTime());
            cyRedisServiceImpl.set(FaceVerifyRedisBean.FACEVERIFY_IDCARD_DAY + idCard, 1, time);
        }
    }

    /**
     * 校验身份证号次数
     * @param memInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    public Object checkIdCard(MemInfo memInfo){
        MemInfo info = baseMapper.getInfoByUserId(String.valueOf(memInfo.getUserId()));
        //验证失败次数
        //获取ip
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String ip = CyAddressUtils.IpUtils.getIpAddr(request);
        //1. 同一IP，每分钟失败次数
        Object captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IP_MINUTES + ip);
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIpMinCount())
                throw new CyServiceException("当前IP实名认证发起频繁，请稍后重试");
        }
        //2. 同一IP，每天失败次数
        captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IP_DAY + ip);
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIpDayCount())
                throw new CyServiceException("当前IP实名认证次数已达上限，请明日再试!");
        }
        //3. 同一身份证，每分钟失败次数
        captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IDCARD_MINUTES + memInfo.getMemIdcard());
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIdCardMinCount())
                throw new CyServiceException("实名认证发起频繁，请稍后重试");
        }
        //4. 同一身份证，每天失败次数
        captcha = cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_IDCARD_DAY + memInfo.getMemIdcard());
        if (captcha != null) {
            int count = (int)captcha;
            if (count >= faceVerifyAliComp.getIdCardDayCount())
                throw new CyServiceException("实名认证次数已达上限，请明日再试!");
        }
        if ("1".equals(info.getMemRealAuthen())){
            // 加次数
            this.faceVerifyFail(memInfo.getMemIdcard());
            throw new CyServiceException("用户已实名，无需再次实名认证");
        }
        cyRedisServiceImpl.set(FaceVerifyRedisBean.FACEVERIFY_USER+memInfo.getUserId(),memInfo,3600L);
        return cyRedisServiceImpl.get(FaceVerifyRedisBean.FACEVERIFY_USER + memInfo.getUserId());
    }
}
