package org.rcisoft.business.memInfo.dao;

import org.rcisoft.business.cmsActivity.entity.CmsActivity;
import org.rcisoft.business.memInfo.entity.*;
import org.rcisoft.business.memTraffic.entity.MemTraffic;
import org.rcisoft.core.mapper.CyBaseMapper;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.sys.rbac.user.dto.SysUserRbacDTO;
import org.rcisoft.sys.rbac.user.entity.SysUserRbac;
import org.springframework.stereotype.Repository;
import org.rcisoft.core.model.CyPageInfo;
import org.apache.ibatis.annotations.Param;
import com.baomidou.mybatisplus.core.metadata.IPage;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;

/**
* Created with cy on 2024年12月20日 下午3:35:34.
*/

public interface MemInfoRepository extends CyBaseMapper<MemInfo> {


    List<MemInfo> queryMemInfos(@Param("entity") MemInfo memInfo);

    /**
    * web端-分页查询 memInfo
    *
    */
    IPage<MemInfo> queryMemInfosPaged(CyPageInfo cyPageInfo,@Param("entity") MemInfo memInfo);

    /**
     * app-分页查询 memInfo
     *
     */
    IPage<MemInfo> appQueryMemInfosPaged(CyPageInfo cyPageInfo,@Param("entity") MemInfo memInfo);

    /**
     * 查询我喜欢的会员列表
     */
    IPage<MemInfo> queryMeFollowPaged(CyPageInfo cyPageInfo,@Param("entity") MemInfo memInfo);

    /**
     * 查询喜欢我的会员列表
     */
    IPage<MemInfo> queryFollowMePaged(CyPageInfo cyPageInfo,@Param("entity") MemInfo memInfo);

    /**
     * 查询互相喜欢的会员列表
     */
    IPage<MemInfo> queryEachFollowPaged(CyPageInfo cyPageInfo,@Param("entity") MemInfo memInfo);

    /**
     * 根据roleKey查询角色
     * @param roleKey
     * @return
     */
    List<Integer> selectRoleByKey(@Param("roleKey")String roleKey);

    /**
     * 校验手机号是否重复
     */
    SysUserRbacDTO checkPhoneRepeat(@Param("userId")Integer userId, @Param("phone")String phone);

    /**
     * 校验身份证是否重复
     */
    SysUserRbacDTO checkIdcardRepeat(@Param("userId")Integer userId, @Param("Idcard")String Idcard);

    /**
     * 查询详情
     * @param businessId
     * @return
     */
    MemInfo selectDetailById(@Param("businessId")int businessId);

    /**
     * 查询最大的会员号
     * @return
     */
    String selectMaxCode();

    /**
     * 查询加密字段
     * @param businessId
     * @return
     */
    MemInfo getUserPhone(Integer businessId);

    IPage<opmArticleDTO> queryOpmArticlesPaged(CyPageInfo cyPageInfo,@Param("entity") opmArticleDTO opmArticleDTO);

    String getRegisterNumber(@Param("entity") peopleNumberDTO peopleNumberDTO);

    String getRealNumber(@Param("entity") peopleNumberDTO peopleNumberDTO);

    String getActivityNumber(@Param("entity") peopleNumberDTO peopleNumberDTO);

    String getArticleNumber(@Param("entity") peopleNumberDTO peopleNumberDTO);

    List<CmsActivity> getCmsActivityList(@Param("entity") peopleNumberDTO peopleNumberDTO);

    List<MemTraffic> getVisitCountList(@Param("entity") peopleNumberDTO peopleNumberDTO);

    List<opmArticleDTO> getArticleCountByDateRangeYes(@Param("startDate") LocalDate startDate,@Param("endDate") LocalDate endDate);

    List<opmArticleDTO> getArticleCountByMonthRangeYes(@Param("startDateMonth")LocalDate startDateMonth,@Param("endDate") LocalDate endDate);

    List<opmArticleDTO> getArticleCountByDateRangeNo(@Param("startDate") LocalDate startDate,@Param("endDate") LocalDate endDate);

    List<opmArticleDTO> getArticleCountByMonthRangeNo(@Param("startDateMonth")LocalDate startDateMonth,@Param("endDate") LocalDate endDate);

    /**
     * 查询加密字段
     * @param userId
     * @return
     */
    MemInfo getInfoByUserId(String userId);

    /**
     * 根据openid查询用户信息
     * @param openId
     * @return
     */
    MemInfo selectByOpenId(String openId);

    /**
     * 根据userId查询用户信息
     * @param userId
     * @return
     */
    MemInfo selectByUserId(String userId);

    /**
     * 关注
     */
    Integer addFollow(@Param("entity") MemFollowDTO followDTO);

    /**
     * 查看是否已关注
     */
    MemFollowDTO getIsFollow(@Param("entity") MemFollowDTO followDTO);

    /**
     * 删除关注
     */
    Integer deleteFollow(@Param("entity") MemFollowDTO followDTO);

    /**
     * 点赞
     */
    Integer addLike(@Param("entity") MemLikeDTO likeDTO);

    /**
     * 点赞表 修改点赞数
     */
    Integer updateLikeCount(@Param("entity") MemLikeDTO likeDTO);

    /**
     * 会员表 点赞数+1
     */
    Integer addMemLikeCount(@Param("entity") MemLikeDTO likeDTO);

    /**
     * 点赞表 点赞数+1
     */
    Integer addLikeCount(@Param("entity") MemLikeDTO likeDTO);

    /**
     * 查询关注的id集合
     */
    List<Integer> queryFollowIdById(@Param("userId") String authenBusinessId);

    /**
     * 排行榜
     */
    IPage<MemInfo> queryRankPaged(CyPageInfo cyPageInfo,@Param("entity") MemInfo memInfo);


    /**
     * 查询我喜欢的  喜欢我的  互相喜欢的数量
     */
    MemInfo selectUserFollowCount(Integer userId);

    /**
     * 分页查询会员热度表
     */
    IPage<MemInfo> queryMemHostPaged(CyPageInfo cyPageInfo,@Param("entity") MemInfo memInfo);

    /**
     * 添加索要微信任务
     */
    int addWeChatTask(@Param("entity") MemUserTaskDTO memUserTaskDTO);

    /**
     * 根据id修改索要微信任务
     */
    int updateWeChatTaskById(@Param("entity") MemUserTaskDTO memUserTaskDTO);

    /**
     * 根据id修改索要微信任务
     */
    int updateWeChatTask(@Param("entity") MemUserTaskDTO memUserTaskDTO);

    /**
     * 查询互换微信任务记录
     */
    MemUserTaskDTO selectWeChatTask(@Param("entity") MemUserTaskDTO memUserTaskDTO);

    /**
     * 查询所有用户id
     */
    List<Integer> selectUserIdList();

    /**
     * 查询当前用户的总被点赞数
     */
    Integer selectUserLikedCount(Integer userId);

    /**
     * 查询当前用户的总被点赞数
     */
    Integer selectUserFollowedCount(Integer userId);

    /**
     * 查询当前对话的未回复的微信任务数量
     */
    int queryTalkWeChatTaskNoReplyCount(@Param("createBy") Integer createBy,@Param("targetId") Integer targetId,@Param("loginUserId") Integer loginUserId);

    /**
     * 查询未回复的微信任务数量
     */
    int queryWeChatTaskNoReplyCount(String userId);

    /**
     * 根据token获取用户信息
     */
    MemInfo getUserInfo(String userId);

    /**
     * 查询当前用户是否已被停用
     */
    String getUserIsStop(String userId);

    /**
     * 根据手机号修改会员openid
     */
    int updateMemWxByPhone(SysUserRbac var1);

    /**
     * 根据手机号修改会员性别
     */
    int updateMemSexByPhone(SysUserRbac var1);


    /**
     * 根据用户id获取用户性别
     */
    String getMemSexByUserId(Integer userId);
}

